@@ -453,6 +453,16 @@ index_header(Fd):-
     format(Fd, '*/~n~n', []).
 
 exports(File, Module, Exports) :-
+    (   current_prolog_flag(xref, Old)
+    ->  true
+    ;   Old = false
+    ),
+    setup_call_cleanup(
+        set_prolog_flag(xref, true),
+        exports_(File, Module, Exports),
+        set_prolog_flag(xref, Old)).
+
+exports_(File, Module, Exports) :-
     State = state(true, _, []),
     (   '$source_term'(File, _,_,Term0,_,_,[syntax_errors(quiet)]),
         (   is_list(Term0)
@@ -597,11 +607,9 @@ autoload_in(Module, How) :-
 
 autoload_in(true,             _,        _).
 autoload_in(explicit,         explicit, _).
-autoload_in(explicit_or_user, explicit, _).
-autoload_in(user,             explicit, user).
-autoload_in(explicit_or_user, explicit, _).
 autoload_in(user,             _,        user).
-autoload_in(explicit_or_user, general,  user).
+autoload_in(user_or_explicit, explicit, _).
+autoload_in(user_or_explicit, _,        user).
 
 
 %!  do_autoload(+File, :PI, +LoadModule) is det.
