@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker and Richard O'Keefe
     E-mail:        J.Wielemaker@cs.vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2002-2021, University of Amsterdam
+    Copyright (c)  2002-2022, University of Amsterdam
                               VU University Amsterdam
                               SWI-Prolog Solutions b.v.
     All rights reserved.
@@ -262,7 +262,8 @@ delete([Elem|Tail], Del, Result) :-
 
 nth0(Index, List, Elem) :-
     (   integer(Index)
-    ->  nth0_det(Index, List, Elem)         % take nth deterministically
+    ->  '$seek_list'(Index, List, RestIndex, RestList),
+        nth0_det(RestIndex, RestList, Elem) % take nth det
     ;   var(Index)
     ->  List = [H|T],
         nth_gen(T, Elem, H, 0, Index)       % match
@@ -270,13 +271,8 @@ nth0(Index, List, Elem) :-
     ).
 
 nth0_det(0, [Elem|_], Elem) :- !.
-nth0_det(1, [_,Elem|_], Elem) :- !.
-nth0_det(2, [_,_,Elem|_], Elem) :- !.
-nth0_det(3, [_,_,_,Elem|_], Elem) :- !.
-nth0_det(4, [_,_,_,_,Elem|_], Elem) :- !.
-nth0_det(5, [_,_,_,_,_,Elem|_], Elem) :- !.
-nth0_det(N, [_,_,_,_,_,_   |Tail], Elem) :-
-    M is N - 6,
+nth0_det(N, [_|Tail], Elem) :-
+    M is N - 1,
     M >= 0,
     nth0_det(M, Tail, Elem).
 
@@ -405,15 +401,17 @@ same_length([_|T1], [_|T2]) :-
 
 %!  reverse(?List1, ?List2)
 %
-%   Is true when the elements of List2 are in reverse order compared to
-%   List1.
+%   Is true when the elements of List2  are in reverse order compared to
+%   List1. This predicate is deterministic if   either  list is a proper
+%   list. If both  lists  are   _partial  lists_  backtracking generates
+%   increasingly long lists.
 
 reverse(Xs, Ys) :-
-    reverse(Xs, [], Ys, Ys).
+    reverse(Xs, Ys, [], Ys).
 
-reverse([], Ys, Ys, []).
-reverse([X|Xs], Rs, Ys, [_|Bound]) :-
-    reverse(Xs, [X|Rs], Ys, Bound).
+reverse([], [], Ys, Ys).
+reverse([X|Xs], [_|Bound], Rs, Ys) :-
+    reverse(Xs, Bound, [X|Rs], Ys).
 
 
 %!  permutation(?Xs, ?Ys) is nondet.
