@@ -528,6 +528,18 @@ test(ieee_rmode) :-
     assertion(test_rounding(pi)),
     assertion(test_rounding(e)).
 
+% rational exponent power tests (3)
+test(float_to_rat) :-
+    assertion(test_value(8.0 is 4.0**3r2)),
+    assertion(1.5NaN is -4.0**3r2),  % neg. base, even Q
+    assertion(test_value(2.0 is 8.0**1r3)),
+    assertion(test_value(-2.0 is -8.0**1r3)),
+    assertion(test_value(4.0 is 8.0**2r3)),
+    assertion(test_value(4.0 is -8.0**2r3)),
+    assertion(test_value(0.5 is 8.0** -1r3)),
+    assertion(test_value(-0.5 is -8.0** -1r3)),
+    assertion(2**1r2 =:= sqrt(2)).
+
 test(ieee_rndto) :-
     assertion(test_roundto(-1.0/(3))),
     assertion(test_roundto(float(1r3))),
@@ -535,8 +547,21 @@ test(ieee_rndto) :-
     assertion(test_roundto(sqrt(2))),
     assertion(test_roundto(exp(log(2)))),
     assertion(test_roundto(2**0.5)),
-    assertion(test_roundto(-2.0** 1r3)),
-    assertion(test_roundto( 2.0** 1r3)),
+    assertion(test_roundto(-2.0002** 2)),
+    assertion(test_roundto( 2.0002** 2)),
+    assertion(test_roundto( 2.0002** 1r2)),
+    assertion(test_roundto(-2.0002** 1r3)),
+    assertion(test_roundto( 2.0002** 1r3)),
+    assertion(test_roundto(-2.0002** 2r3)),
+    assertion(test_roundto( 2.0002** 2r3)),
+    assertion(test_roundto( 0.0002 ** 1r3)),
+    assertion(test_roundto(-0.0002 ** 1r3)),
+    assertion(test_roundto( 0.0002 ** 2r3)),
+    assertion(test_roundto(-0.0002 ** 2r3)),        
+    assertion(test_roundto( 0.0002 ** -1r3)),
+    assertion(test_roundto(-2.0002 ** -1r3)),
+    assertion(test_roundto( 0.0002 ** -2r3)),
+    assertion(test_roundto(-2.0002 ** -2r3)),
     assertion(test_roundto(pi)),
     assertion(test_roundto(e)),
     assertion(test_roundto(sin(pi/2))),
@@ -581,6 +606,10 @@ float_parts(F,Ip,Fp) :-
     Ip is float_integer_part(F),
     Fp is float_fractional_part(F).
 
+test_value(R is Exp) :-
+	Rp is roundtoward(Exp,to_positive),
+    Rn is roundtoward(Exp,to_negative),
+    Rn =< R,R =< Rp.
 
 test_rounding(Exp) :-
     rounding(Exp, Rounded),
@@ -612,7 +641,7 @@ check_round(_Exp, r(R,R,R,R)) :-
 	float(R), float_class(R,infinite),
 	!.
 check_round(Exp, r(Rc,Rp,Rn,Rz)) :-
-	abs(Rp-Rc+Rn-Rz) < 1e-15,  % all values should be almost equal so sum~=0
+    abs((Rp-Rc+Rn-Rz)/Rc) < 4e-15,
     Rn =< Rc, Rc =< Rp,
     (   Rc < 0
     ->  Rz >= Rc, expect_less_then(Exp, n=Rn, z=Rz)
