@@ -377,21 +377,22 @@ match_pattern(matchcode *p, const char *s, int flags)
 /** wildcard_match(+Pattern, +Name [, +Options]) is semidet.
 */
 
-static const opt_spec wildcard_options[] =
+static const PL_option_t wildcard_options[] =
 { { ATOM_case_sensitive,    OPT_BOOL },
   { NULL_ATOM,		    0 }
 };
 
 
+#define wildcard_match(pattern, string, options) LDFUNC(wildcard_match, pattern, string, options)
 static int
-wildcard_match(term_t pattern, term_t string, term_t options ARG_LD)
+wildcard_match(DECL_LD term_t pattern, term_t string, term_t options)
 { char *p, *s;
   int rc = FALSE;
   int mflags = 0;
   int case_sensitive = TRUE;
 
   if ( options &&
-       !scan_options(options, 0, ATOM_wildcard_option,
+       !PL_scan_options(options, 0, "wildcard_option",
 		     wildcard_options, &case_sensitive) )
     return FALSE;
   if ( !case_sensitive )
@@ -416,14 +417,14 @@ static
 PRED_IMPL("wildcard_match", 2, wildcard_match, 0)
 { PRED_LD
 
-  return wildcard_match(A1, A2, 0 PASS_LD);
+  return wildcard_match(A1, A2, 0);
 }
 
 static
 PRED_IMPL("wildcard_match", 3, wildcard_match, 0)
 { PRED_LD
 
-  return wildcard_match(A1, A2, A3 PASS_LD);
+  return wildcard_match(A1, A2, A3);
 }
 
 
@@ -471,8 +472,9 @@ free_expand_info(GlobInfo info)
 
 
 #ifndef __WINDOWS__
+#define mb_add_path(path, info) LDFUNC(mb_add_path, path, info)
 static void
-mb_add_path(const char *path, GlobInfo info ARG_LD)
+mb_add_path(DECL_LD const char *path, GlobInfo info)
 { int idx = (int)entriesBuffer(&info->strings, char);
   PL_chars_t txt;
 
@@ -482,7 +484,7 @@ mb_add_path(const char *path, GlobInfo info ARG_LD)
   txt.encoding  = ENC_ANSI;
   txt.storage   = PL_CHARS_HEAP;
   txt.canonical = FALSE;
-  if ( PL_canonicalise_text(&txt) &&
+  if ( (PL_canonicalise_text(&txt) == TRUE) &&
        PL_mb_text(&txt, REP_UTF8) )
   { addMultipleBuffer(&info->strings, txt.text.t, txt.length+1, char);
     addBuffer(&info->files, idx, int);
@@ -532,8 +534,9 @@ un_escape(char *to, const char *from, const char *end)
 }
 
 
+#define utf8_exists_file(name) LDFUNC(utf8_exists_file, name)
 static int
-utf8_exists_file(const char *name ARG_LD)
+utf8_exists_file(DECL_LD const char *name)
 {
 #ifndef __WINDOWS__
   PL_chars_t txt;
@@ -545,7 +548,7 @@ utf8_exists_file(const char *name ARG_LD)
   txt.encoding  = ENC_UTF8;
   txt.storage   = PL_CHARS_HEAP;
   txt.canonical = FALSE;
-  rc = ( PL_canonicalise_text(&txt) &&
+  rc = ( (PL_canonicalise_text(&txt) == TRUE) &&
 	 PL_mb_text(&txt, REP_FN) &&
 	 AccessFile(txt.text.t, ACCESS_EXIST) );
   PL_free_text(&txt);
@@ -557,8 +560,9 @@ utf8_exists_file(const char *name ARG_LD)
 #endif
 }
 
+#define utf8_opendir(name) LDFUNC(utf8_opendir, name)
 static DIR*
-utf8_opendir(const char *name ARG_LD)
+utf8_opendir(DECL_LD const char *name)
 {
 #ifndef __WINDOWS__
   PL_chars_t txt;
@@ -570,7 +574,7 @@ utf8_opendir(const char *name ARG_LD)
   txt.encoding  = ENC_UTF8;
   txt.storage   = PL_CHARS_HEAP;
   txt.canonical = FALSE;
-  if ( PL_canonicalise_text(&txt) &&
+  if ( (PL_canonicalise_text(&txt) == TRUE) &&
        PL_mb_text(&txt, REP_FN) )
     rc = opendir(txt.text.t);
   else
@@ -588,8 +592,8 @@ static int
 expand(const char *pattern, GlobInfo info)
 { GET_LD
   const char *pat = pattern;
-  char prefix[MAXPATHLEN];		/* before first pattern */
-  char patbuf[MAXPATHLEN];		/* pattern buffer */
+  char prefix[PATH_MAX];		/* before first pattern */
+  char patbuf[PATH_MAX];		/* pattern buffer */
   size_t prefix_len;
   int end, dot;
   int mflags = 0;
@@ -621,16 +625,16 @@ expand(const char *pattern, GlobInfo info)
 
 	    end = info->end;
 	    for( ; info->start < end; info->start++ )
-	    { char path[MAXPATHLEN];
+	    { char path[PATH_MAX];
 	      const char *entry = expand_entry(info, info->start);
 	      size_t plen = strlen(entry);
 
-	      if ( plen+prefix_len+2 <= MAXPATHLEN )
+	      if ( plen+prefix_len+2 <= PATH_MAX )
 	      { strcpy(path, entry);
 		if ( prefix[0] && plen > 0 && path[plen-1] != '/' )
 		  path[plen++] = '/';
 		strcpy(&path[plen], prefix);
-		if ( end == 1 || utf8_exists_file(path PASS_LD) )
+		if ( end == 1 || utf8_exists_file(path) )
 		  utf8_add_path(path, info);
 	      }
 	    }
@@ -679,8 +683,8 @@ expand(const char *pattern, GlobInfo info)
     for(; info->start < end; info->start++)
     { DIR *d;
       struct dirent *e;
-      char path[MAXPATHLEN];
-      char tmp[MAXPATHLEN];
+      char path[PATH_MAX];
+      char tmp[PATH_MAX];
       const char *current = expand_entry(info, info->start);
       size_t clen = strlen(current);
 
@@ -690,7 +694,7 @@ expand(const char *pattern, GlobInfo info)
       strcpy(path, current);
       strcpy(&path[clen], prefix);
 
-      if ( (d=utf8_opendir((path[0] ? OsPath(path, tmp) : ".") PASS_LD)) )
+      if ( (d=utf8_opendir((path[0] ? OsPath(path, tmp) : "."))) )
       { size_t plen = clen+prefix_len;
 
 	if ( plen > 0 && path[plen-1] != '/' )
@@ -703,7 +707,7 @@ expand(const char *pattern, GlobInfo info)
 #endif
 	  if ( (dot || e->d_name[0] != '.') &&
 	       matchPattern(e->d_name, &info->pattern, mflags) )
-	  { char newp[MAXPATHLEN];
+	  { char newp[PATH_MAX];
 
 	    if ( plen+strlen(e->d_name)+1 < sizeof(newp) )
 	    { strcpy(newp, path);
@@ -711,7 +715,7 @@ expand(const char *pattern, GlobInfo info)
 #ifdef __WINDOWS__			/* readdir() emulation emits UTF-8 */
 	      utf8_add_path(newp, info);
 #else
-	      mb_add_path(newp, info PASS_LD);
+	      mb_add_path(newp, info);
 #endif
 	    }
 	  }
@@ -759,7 +763,7 @@ sort_expand(GlobInfo info)
 static
 PRED_IMPL("expand_file_name", 2, expand_file_name, 0)
 { PRED_LD
-  char spec[MAXPATHLEN];
+  char spec[PATH_MAX];
   char *s;
   glob_info info;
   term_t l    = PL_copy_term_ref(A2);
