@@ -274,6 +274,7 @@ clock_jitter(double t)
 double
 CpuTime(cputime_kind which)
 {
+#ifndef __EMSCRIPTEN__
 #if defined(HAVE_CLOCK_GETTIME) && defined(CLOCK_PROCESS_CPUTIME_ID)
 #define CPU_TIME_DONE
   struct timespec ts;
@@ -311,10 +312,14 @@ CpuTime(cputime_kind which)
 
   return clock_jitter(used);
 #endif
+#endif /* not __EMSCRIPTEN__ */
 
 #if !defined(CPU_TIME_DONE)
+  GET_LD
   (void)which;
 
+  return WallTime() - LD->statistics.start_time;
+
   return 0.0;
 #endif
 }
@@ -2155,15 +2160,12 @@ Sread_terminal(void *handle, char *buf, size_t size)
   int fd = (int)h;
   source_location oldsrc = LD->read_source;
 
-  if ( Soutput )
-  { if ( LD->prompt.next &&
-	 Sinput &&
-	 false(Sinput, SIO_RAW) &&
-	 true(Sinput, SIO_ISATTY) )
-      PL_write_prompt(TRUE);
-    else if ( true(Soutput, SIO_ISATTY) )
-      Sflush(Suser_output);
-  }
+  if ( LD->prompt.next &&
+       false(Sinput, SIO_RAW) &&
+       true(Sinput, SIO_ISATTY) )
+    PL_write_prompt(TRUE);
+  else if ( true(Soutput, SIO_ISATTY) )
+    Sflush(Suser_output);
 
   PL_dispatch(fd, PL_DISPATCH_WAIT);
   size = (*GD->os.org_terminal.read)(handle, buf, size);
