@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2011-2021, University of Amsterdam
+    Copyright (c)  2011-2022, University of Amsterdam
                               VU University Amsterdam
 			      CWI, Amsterdam
 			      SWI-Prolog Solutions b.v.
@@ -42,6 +42,7 @@
 #include <process.h>			/* getpid() */
 #endif
 #include "pl-prologflag.h"
+#include "pl-utf8.h"
 #include "pl-ctype.h"
 #include "../pl-arith.h"
 #include "../pl-tabling.h"
@@ -988,7 +989,7 @@ PRED_IMPL("set_prolog_flag", 2, set_prolog_flag, PL_FA_ISO)
 /** create_prolog_flag(+Key, +Value, +Options) is det.
 */
 
-static const opt_spec prolog_flag_options[] =
+static const PL_option_t prolog_flag_options[] =
 { { ATOM_type,   OPT_ATOM },
   { ATOM_access, OPT_ATOM },
   { ATOM_keep,   OPT_BOOL },
@@ -1003,7 +1004,7 @@ PRED_IMPL("create_prolog_flag", 3, create_prolog_flag, PL_FA_ISO)
   atom_t access = ATOM_read_write;
   int keep = FALSE;
 
-  if ( !scan_options(A3, 0, ATOM_prolog_flag_option, prolog_flag_options,
+  if ( !PL_scan_options(A3, 0, "prolog_flag_option", prolog_flag_options,
 		     &type, &access, &keep) )
     return FALSE;
 
@@ -1499,6 +1500,7 @@ initPrologFlags(void)
   setPrologFlag("trace_gc",  FT_BOOL,	       FALSE, PLFLAG_TRACE_GC);
 #ifdef O_ATOMGC
   setPrologFlag("agc_margin",FT_INTEGER,	       GD->atoms.margin);
+  setPrologFlag("agc_close_streams", FT_BOOL, FALSE, PLFLAG_AGC_CLOSE_STREAMS);
 #endif
   setPrologFlag("table_space", FT_INTEGER, GD->options.tableSpace);
 #ifdef O_PLMT
@@ -1569,7 +1571,7 @@ initPrologFlags(void)
     setPrologFlag("integer_rounding_function", FT_ATOM|FF_READONLY, "down");
   else
     setPrologFlag("integer_rounding_function", FT_ATOM|FF_READONLY, "toward_zero");
-  setPrologFlag("max_char_code", FT_INTEGER|FF_READONLY, PLMAXWCHAR);
+  setPrologFlag("max_char_code", FT_INTEGER|FF_READONLY, UNICODE_MAX);
   setPrologFlag("max_arity", FT_ATOM|FF_READONLY, "unbounded");
   setPrologFlag("max_procedure_arity", FT_INTEGER|FF_READONLY, MAXARITY);
   setPrologFlag("answer_format", FT_ATOM, "~p");
