@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2021, University of Amsterdam,
+    Copyright (c)  1985-2022, University of Amsterdam,
                               VU University Amsterdam
 			      CWI, Amsterdam
 			      SWI-Prolog Solutions b.v.
@@ -41,45 +41,12 @@
 #define PLNAME "swi"
 
 #ifdef __WINDOWS__
-#ifdef WIN64
-#include "config/win64.h"
-#define PLHOME       "c:/Program Files/swipl"
-#else
-#include "config/win32.h"
-#define PLHOME       "c:/Program Files (x86)/swipl"
-#endif
+#include "config/wincfg.h"
 #else /*__WINDOWS__*/
 #include <config.h>
 #endif
 
-#ifdef _MSC_VER
-#define C_LIBS	     ""
-#define C_STATICLIBS ""
-#define C_CC	     "cl"
-#if (_MSC_VER < 1400)
-#define C_CFLAGS     "/MD /GX"
-#else
-#define C_CFLAGS     "/MD /EHsc"
-#endif
-#define C_LDFLAGS    ""
-#if defined(_DEBUG)
-#define C_PLLIB	    "swiplD.lib"
-#else
-#define C_PLLIB	    "swipl.lib"
-#endif
-#else					/* !_MSC_VER  */
-#ifdef __WINDOWS__			/* I.e., MinGW */
-#define C_LIBS	     ""
-#define C_STATICLIBS ""
-#define C_CC	     "gcc"
-#define C_CFLAGS     ""
-#define C_PLLIB	     "-lswipl"		/* Or "libswipl.lib"? */
-#define C_LIBPLSO    "-lswipl"
-#define C_LDFLAGS    ""
-#else
 #include <parms.h>			/* pick from the working dir */
-#endif
-#endif
 
 /* gmp.h must be included PRIOR to SWI-Prolog.h to enable the API prototypes */
 #ifdef HAVE_GMP_H
@@ -214,6 +181,14 @@ handy for it someone wants to add a data type to the system.
       virtual Prolog "signals" defined above SIG_PROLOG_OFFSET, but
       PL_thread_raise loses its "interrupt a blocking system call"
       semantics when this is disabled.
+  O_ROUND_UP_DOWN
+      Force true values for floating point functions are between the
+      downward and upward rounded values.  Works around the fact that
+      most C runtime libraries do not correctly implement the float
+      rouding modes for many of the trigonomy and expononential
+      functions.
+  O_COVERAGE
+      Include low-level coverage analysis code.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 #define O_COMPILE_OR		1
@@ -241,6 +216,8 @@ handy for it someone wants to add a data type to the system.
 #define O_GVAR			1
 #define O_CYCLIC		1
 #define O_MITIGATE_SPECTRE	1
+#define O_ROUND_UP_DOWN		1
+#define O_COVERAGE		1
 #ifndef O_PREFER_RATIONALS
 #define O_PREFER_RATIONALS	FALSE
 #endif
@@ -435,14 +412,14 @@ typedef _sigset_t sigset_t;
 #define EMULATE_DLOPEN 1		/* Emulated dlopen() in pl-beos.c */
 #endif
 
-/* MAXPATHLEN is an optional POSIX feature (Bug#63).  As SWI-Prolog has
+/* PATH_MAX is an optional POSIX feature (Bug#63).  As SWI-Prolog has
    no length limits on text except for representing paths, we should
-   rewrite all file handling code to avoid MAXPATHLEN.  For now we just
+   rewrite all file handling code to avoid PATH_MAX.  For now we just
    define it.
 */
 
-#ifndef MAXPATHLEN
-#define MAXPATHLEN 1024
+#ifndef PATH_MAX
+#define PATH_MAX 1024
 #endif
 
 /* If we have a threads-supporting C11 environment, we can use C11 thread
@@ -597,11 +574,6 @@ them.  Descriptions:
 #define inTaggedNumRange(n)	(valInt(consInt(n)) == (n))
 #define PLMININT		(-PLMAXINT - 1)
 #define PLMAXINT		((int64_t)(((uint64_t)1<<(INT64BITSIZE-1)) - 1))
-#if SIZEOF_WCHAR_T == 2
-#define PLMAXWCHAR		(0xffff)
-#else
-#define PLMAXWCHAR		(0x10ffff)
-#endif
 
 #if vax
 #define MAXREAL			(1.701411834604692293e+38)
@@ -932,6 +904,7 @@ typedef enum
 #define	ALERT_DEBUG	     0x080
 #define	ALERT_BUFFER	     0x100
 #define	ALERT_UNDO	     0x200
+#define ALERT_COVERAGE	     0x400
 
 
 		 /*******************************
@@ -2426,6 +2399,8 @@ typedef struct
 	  if ( save ) \
 	  { _savedf = LD->var_names.numbervars_frame; \
 	    LD->var_names.numbervars_frame = PL_open_foreign_frame(); \
+	  } else \
+	  { _savedf = 0; /* avoid (false) uninitialized */  \
 	  }
 #define END_NUMBERVARS(save) \
           if ( save ) \
@@ -2653,6 +2628,7 @@ typedef enum plflag
   PLFLAG_AUTOLOAD,			/* do autoloading */
   PLFLAG_CHARCONVERSION,		/* do character-conversion */
   PLFLAG_LASTCALL,			/* Last call optimization enabled? */
+  PLFLAG_VMI_BUILTIN,			/* Use VMI for simple built-ins */
   PLFLAG_PORTABLE_VMI,			/* Generate portable VMI code */
   PLFLAG_SIGNALS,			/* Handle signals */
   PLFLAG_DEBUGINFO,			/* generate debug info */
@@ -2668,7 +2644,8 @@ typedef enum plflag
   PLFLAG_RATIONAL,			/* Natural rational numbers */
   PLFLAG_DEBUG_ON_INTERRUPT,		/* Debug on Control-C */
   PLFLAG_OPTIMISE_UNIFY,		/* Move unifications in clauses */
-  PLFLAG_SHIFT_CHECK			/* Check suspicious shifts */
+  PLFLAG_SHIFT_CHECK,			/* Check suspicious shifts */
+  PLFLAG_AGC_CLOSE_STREAMS		/* AGC may close open streams */
 } plflag;
 
 typedef struct
@@ -2686,6 +2663,18 @@ typedef struct
 	ATOMIC_AND(&prologFlagMaskInt(LD, flag), ~prologFlagMask(flag))
 #define setPrologFlagMask(flag) setPrologFlagMask_LD(LD, flag)
 
+#define RUN_MODE_NORMAL \
+	(prologFlagMask(PLFLAG_LASTCALL)| \
+	 prologFlagMask(PLFLAG_VMI_BUILTIN))
+#define setPrologRunMode_LD(ld, mask) \
+	ATOMIC_OR(&prologFlagMaskInt(ld, PLFLAG_LASTCALL), mask)
+#define clearPrologRunMode_LD(ld, mask) \
+	ATOMIC_AND(&prologFlagMaskInt(ld, PLFLAG_LASTCALL), ~(mask))
+#define setPrologRunMode(mask) \
+	setPrologRunMode_LD(LD, mask)
+#define clearPrologRunMode(mask) \
+	clearPrologRunMode_LD(LD, mask)
+
 typedef enum
 { OCCURS_CHECK_FALSE = 0,	/* allow rational trees */
   OCCURS_CHECK_TRUE,		/* fail if rational tree would result */
